@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2015, 2016 OpenMarket Ltd
 # Copyright 2017 Vector Creations Ltd
 # Copyright 2018-2019 New Vector Ltd
@@ -18,15 +17,12 @@
 
 import jsonschema
 
-from twisted.internet import defer
-
 from synapse.api.constants import EventContentFields
 from synapse.api.errors import SynapseError
 from synapse.api.filtering import Filter
 from synapse.events import make_event_from_dict
 
 from tests import unittest
-from tests.utils import setup_test_homeserver
 
 user_localpart = "test_user"
 
@@ -39,9 +35,8 @@ def MockEvent(**kwargs):
     return make_event_from_dict(kwargs)
 
 
-class FilteringTestCase(unittest.TestCase):
-    def setUp(self):
-        hs = setup_test_homeserver(self.addCleanup)
+class FilteringTestCase(unittest.HomeserverTestCase):
+    def prepare(self, reactor, clock, hs):
         self.filtering = hs.get_filtering()
         self.datastore = hs.get_datastore()
 
@@ -351,10 +346,9 @@ class FilteringTestCase(unittest.TestCase):
 
         self.assertTrue(Filter(definition).check(event))
 
-    @defer.inlineCallbacks
     def test_filter_presence_match(self):
         user_filter_json = {"presence": {"types": ["m.*"]}}
-        filter_id = yield defer.ensureDeferred(
+        filter_id = self.get_success(
             self.datastore.add_user_filter(
                 user_localpart=user_localpart, user_filter=user_filter_json
             )
@@ -362,7 +356,7 @@ class FilteringTestCase(unittest.TestCase):
         event = MockEvent(sender="@foo:bar", type="m.profile")
         events = [event]
 
-        user_filter = yield defer.ensureDeferred(
+        user_filter = self.get_success(
             self.filtering.get_user_filter(
                 user_localpart=user_localpart, filter_id=filter_id
             )
@@ -371,11 +365,10 @@ class FilteringTestCase(unittest.TestCase):
         results = user_filter.filter_presence(events=events)
         self.assertEquals(events, results)
 
-    @defer.inlineCallbacks
     def test_filter_presence_no_match(self):
         user_filter_json = {"presence": {"types": ["m.*"]}}
 
-        filter_id = yield defer.ensureDeferred(
+        filter_id = self.get_success(
             self.datastore.add_user_filter(
                 user_localpart=user_localpart + "2", user_filter=user_filter_json
             )
@@ -387,7 +380,7 @@ class FilteringTestCase(unittest.TestCase):
         )
         events = [event]
 
-        user_filter = yield defer.ensureDeferred(
+        user_filter = self.get_success(
             self.filtering.get_user_filter(
                 user_localpart=user_localpart + "2", filter_id=filter_id
             )
@@ -396,10 +389,9 @@ class FilteringTestCase(unittest.TestCase):
         results = user_filter.filter_presence(events=events)
         self.assertEquals([], results)
 
-    @defer.inlineCallbacks
     def test_filter_room_state_match(self):
         user_filter_json = {"room": {"state": {"types": ["m.*"]}}}
-        filter_id = yield defer.ensureDeferred(
+        filter_id = self.get_success(
             self.datastore.add_user_filter(
                 user_localpart=user_localpart, user_filter=user_filter_json
             )
@@ -407,7 +399,7 @@ class FilteringTestCase(unittest.TestCase):
         event = MockEvent(sender="@foo:bar", type="m.room.topic", room_id="!foo:bar")
         events = [event]
 
-        user_filter = yield defer.ensureDeferred(
+        user_filter = self.get_success(
             self.filtering.get_user_filter(
                 user_localpart=user_localpart, filter_id=filter_id
             )
@@ -416,10 +408,9 @@ class FilteringTestCase(unittest.TestCase):
         results = user_filter.filter_room_state(events=events)
         self.assertEquals(events, results)
 
-    @defer.inlineCallbacks
     def test_filter_room_state_no_match(self):
         user_filter_json = {"room": {"state": {"types": ["m.*"]}}}
-        filter_id = yield defer.ensureDeferred(
+        filter_id = self.get_success(
             self.datastore.add_user_filter(
                 user_localpart=user_localpart, user_filter=user_filter_json
             )
@@ -429,7 +420,7 @@ class FilteringTestCase(unittest.TestCase):
         )
         events = [event]
 
-        user_filter = yield defer.ensureDeferred(
+        user_filter = self.get_success(
             self.filtering.get_user_filter(
                 user_localpart=user_localpart, filter_id=filter_id
             )
@@ -454,11 +445,10 @@ class FilteringTestCase(unittest.TestCase):
 
         self.assertEquals(filtered_room_ids, ["!allowed:example.com"])
 
-    @defer.inlineCallbacks
     def test_add_filter(self):
         user_filter_json = {"room": {"state": {"types": ["m.*"]}}}
 
-        filter_id = yield defer.ensureDeferred(
+        filter_id = self.get_success(
             self.filtering.add_user_filter(
                 user_localpart=user_localpart, user_filter=user_filter_json
             )
@@ -468,7 +458,7 @@ class FilteringTestCase(unittest.TestCase):
         self.assertEquals(
             user_filter_json,
             (
-                yield defer.ensureDeferred(
+                self.get_success(
                     self.datastore.get_user_filter(
                         user_localpart=user_localpart, filter_id=0
                     )
@@ -476,17 +466,16 @@ class FilteringTestCase(unittest.TestCase):
             ),
         )
 
-    @defer.inlineCallbacks
     def test_get_filter(self):
         user_filter_json = {"room": {"state": {"types": ["m.*"]}}}
 
-        filter_id = yield defer.ensureDeferred(
+        filter_id = self.get_success(
             self.datastore.add_user_filter(
                 user_localpart=user_localpart, user_filter=user_filter_json
             )
         )
 
-        filter = yield defer.ensureDeferred(
+        filter = self.get_success(
             self.filtering.get_user_filter(
                 user_localpart=user_localpart, filter_id=filter_id
             )
